-----------------------------------
-- Henge - Generic Necrosis-like --
-----------------------------------

Henge = AceLibrary("AceAddon-2.0"):new(
	"AceConsole-2.0",
	"AceDB-2.0",
	"AceDebug-2.0",
	"AceEvent-2.0",
	"AceModuleCore-2.0"
)
local L  = AceLibrary("AceLocale-2.2"):new("Henge")

local AceOO = AceLibrary("AceOO-2.0")
local Dewdrop = AceLibrary("Dewdrop-2.0")
local Tablet = AceLibrary("Tablet-2.0")
local PT = AceLibrary("PeriodicTable-3.0")

local classNames = {
	["WARRIOR"] = "RageHenge",
	["HUNTER"]  = "BeastHenge",
	["ROGUE"]   = "StealthHenge",
	["PRIEST"]  = "FaithHenge",
	["MAGE"]    = "RuneHenge",
	["DRUID"]   = "TreeHenge",
	["SHAMAN"]  = "TotemHenge",
	["PALADIN"] = "BubbleHenge",
	["WARLOCK"] = "StoneHenge"
}

local eventFuncs = {
	["LearnSpell"] = {},
	["BagSlotUpdate"] = {}
}

local defaults = {
	["profile"] = {
		["locked"] = false
	},
	["char"] = {
		["actopt"] = {}
	}
}

local spellInfo = {}
local itemInfo = {}
local loadedModules = {}

function Henge:OnInitialize()
	self:SetDebugging(true)
	self.info = {
		["name"] = "Henge",
		["icon"] = "Interface\\AddOns\\Henge\\icon"
	}
	local _, myClass = UnitClass("player")
	for loop = 1, GetNumAddOns(), 1 do
		local loadIt = true
		local xHengeModule = GetAddOnMetadata(loop, "X-Henge-Module")
		if not xHengeModule then
			loadIt = false
		else
			local xHengeClass = GetAddOnMetadata(loop, "X-Henge-Class")
			if xHengeClass then
				if strupper(xHengeClass) ~= self.myClass then
					loadIt = false
				end
			end
		end
		if loadIt == true and IsAddOnLoadOnDemand(loop) then
			LoadAddOn(loop)
		end
	end
	for mname, module in self:IterateModules() do
		if self:IsModuleActive(module) then
			if (not type(module.Test) == "function") or module:Test() then
				loadedModules[mname] = module
			end
		end
	end
	self:RegisterDB("HengeDB", "HengePCDB")
	self:RegisterEvent("SpecialEvents_LearnSpell")
	self:RegisterEvent("SpecialEvents_BagSlotUpdate")
	self:RegisterEvent("PLAYER_LOGIN")
end

function Henge:PLAYER_LOGIN()
	HengeSphere:AddData({
		["id"]   = "none",
		["name"] = L["None"],
		["func"] = function() return 0, 0, false end,
		["event"] = nil
	})
	HengeSphere:AddData({
		["id"]   = "health",
		["name"] = L["Health"],
		["func"] = function()
			return UnitHealth("player"), UnitHealthMax("player"), false
		end,
		["event"] = "UNIT_HEALTH",
		["arg1"] = "player"
	})
	HengeAction:Add({
		["id"]   = "none",
		["type"] = 'none',
		["name"] = L["None"]
	})
	self:RegisterDefaults("profile", defaults.profile)
	self:RegisterDefaults("char", defaults.char)
	if self.db.profile.config_version ~= 3 then
		self:ResetDB("profile")
		self.db.profile.config_version = 3
	end
	if self.db.char.config_version ~= 3 then
		self:ResetDB("char")
		self.db.char.config_version = 3
	end
	HengeSphere:Add({["id"] = "Main", ["name"] = "Main"})
	local _, myClass = UnitClass("player")
	for mname, module in pairs(loadedModules) do
		if mname == classNames[myClass] then
			local path = "Interface\\AddOns\\"
			if IsAddOnLoaded(mname) then
				path = path..mname.."\\"
			else
				path = path.."Henge\\Modules\\"..mname.."\\"
			end
			self.info.name = mname
			self.info.icon = path..module.icon
			module.path = path
		end
		module:Load()
	end
	local optTable = {
		["type"] = 'group',
		["name"] = self.info.name,
		["icon"] = self.info.icon,
		["args"] = {
			["lock"] = {
				["type"] = 'toggle',
				["name"] = L["Lock Frames"],
				["desc"] = L["Prevent moving of the spheres and buttons."],
				["get"]  = function() return Henge:IsLocked() end,
				["set"]  = function() Henge:ToggleLocked() end,
				["order"] = 6
			},
			["update"] = {
				["type"] = 'execute',
				["name"] = L["Update Frames"],
				["desc"] = L["Force Henge to update its frames."],
				["func"] = function() Henge:UpdateAll() end,
				["order"] = 9
			},
			["reset"] = {
				["type"] = 'execute',
				["name"] = L["Reset Defaults"],
				["desc"] = L["Restore the default settings."],
				["func"] = function() Henge:ResetDefaults() end,
				["order"] = -2
			}
		}
	}
	self:RegisterChatCommand({"/henge"}, optTable)
	self:UpdateAll()
end

function Henge.modulePrototype:SetupDB(defs)
	self.db = Henge:AcquireDBNamespace(self.fullname)
	for k, v in pairs(defs) do
		Henge:RegisterDefaults(self.fullname, k, v)
	end
end

function Henge:GetThemeBase()
	return "Interface\\AddOns\\Henge\\images\\"
end

function Henge:ShowConfig()
end

function Henge:ResetDefaults()
	self:ResetDB("profile")
	self:ResetDB("char")
	ReloadUI()
end

function Henge:IsLocked()
	return self.db.profile.locked ~= false
end

function Henge:ToggleLocked()
	if self.db.profile.locked ~= false then
		self.db.profile.locked = false
	else
		self.db.profile.locked = true
	end
end

function Henge:UpdateAll()
	HengeSphere:UpdateAll()
	HengeButton:UpdateAll()
end

function Henge:SpecialEvents_LearnSpell()
	for k, v in pairs(eventFuncs.LearnSpell) do
		v()
	end
end

function Henge:SpecialEvents_BagSlotUpdate(bag, slot, itemlink, stack)
	for k, v in pairs(eventFuncs.BagSlotUpdate) do
		v()
	end
end

function Henge:LearnSpellFunc(id, func)
	eventFuncs.LearnSpell[id] = func
end

function Henge:BagSlotUpdateFunc(id, func)
	eventFuncs.BagSlotUpdate[id] = func
end
